<?php

namespace Drupal\entity_data;

use Drupal\Core\Database\Connection;

/**
 * Defines the entity data service.
 */
class EntityData implements EntityDataInterface {

  /**
   * The database connection to use.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $connection;

  /**
   * Constructs a new entity data service.
   *
   * @param \Drupal\Core\Database\Connection $connection
   *   The database connection to use.
   */
  public function __construct(Connection $connection) {
    $this->connection = $connection;
  }

  /**
   * {@inheritdoc}
   */
  public function get($module, $entity_id = NULL, $name = NULL, $entity_type = NULL) {
    $query = $this->connection->select('entity_data', 'ed')
      ->fields('ed')
      ->condition('module', $module);

    if (!empty($entity_id)) {
      $query->condition('entity_id', $entity_id);
    }

    if (!empty($name)) {
      $query->condition('name', $name);
    }

    if (!empty($entity_type)) {
      $query->condition('entity_type', $entity_type);
    }

    $result = $query->execute();

    // If $module, $entity_id, and $name were passed, return the value.
    if (isset($name) && isset($entity_id)) {
      $result = $result->fetchAllAssoc('entity_id');
      if (isset($result[$entity_id])) {
        return $result[$entity_id]->serialized ? unserialize($result[$entity_id]->value) : $result[$entity_id]->value;
      }
      return NULL;
    }

    $return = [];
    // If $module and $entity_id were passed, return data keyed by name.
    if (isset($entity_id)) {
      foreach ($result as $record) {
        $return[$record->name] = ($record->serialized ? unserialize($record->value) : $record->value);
      }
      return $return;
    }

    // If $module and $name were passed, return data keyed by entity_id.
    if (isset($name)) {
      foreach ($result as $record) {
        $return[$record->entity_id] = ($record->serialized ? unserialize($record->value) : $record->value);
      }
      return $return;
    }

    // If only $module was passed, return data keyed by entity_id and name.
    foreach ($result as $record) {
      $return[$record->entity_id][$record->name] = ($record->serialized ? unserialize($record->value) : $record->value);
    }

    return $return;
  }

  /**
   * {@inheritdoc}
   */
  public function set($module, $entity_id, $name, $entity_type, $value) {
    $serialized = (int) !is_scalar($value);
    if ($serialized) {
      $value = serialize($value);
    }
    $this->connection->merge('entity_data')
      ->keys([
        'entity_id' => $entity_id,
        'module' => $module,
        'entity_type' => $entity_type,
        'name' => $name,
      ])
      ->fields([
        'value' => $value,
        'serialized' => $serialized,
      ])
      ->execute();
  }

  /**
   * {@inheritdoc}
   */
  public function delete($module = NULL, $entity_id = NULL, $name = NULL, $entity_type = NULL) {
    $query = $this->connection->delete('entity_data');

    // Cast scalars to array so we can consistently use an IN condition.
    if (!empty($module)) {
      $query->condition('module', (array) $module, 'IN');
    }

    if (!empty($entity_id)) {
      $query->condition('entity_id', (array) $entity_id, 'IN');
    }

    if (!empty($name)) {
      $query->condition('name', (array) $name, 'IN');
    }

    if (!empty($entity_type)) {
      $query->condition('entity_type', (array) $entity_type, 'IN');
    }

    $query->execute();
  }

}
